import os
import sys
import logging

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')] + sys.path

def validate_input(arguments):
    check_illumina_pe(arguments.illumina)
    check_database(arguments.database, arguments.reference)

def check_illumina_pe(files):
    if len(files) % 2 != 0:
        logging.info('Error: Illumina files must be paired-end. Exiting. An odd number of files was provided.')
        sys.exit('Error: Illumina files must be paired-end. Exiting.')

def check_database(database, reference):
    if reference == None:
        db_list = ['.name', '.comp.b', '.length.b', '.seq.b']
        for db in db_list:
            if not os.path.isfile(database + db):
                logging.info('Error: KMA database not found. Exiting.')
                sys.exit('Error: KMA database not found. Exiting.')

def check_for_ccphylo():
    """Checks if ccphylo is installed"""
    try:
        subprocess.call(["ccphylo"], stdout=open(os.devnull, 'wb'))
    except FileNotFoundError:
        self.logger.info("ccphylo is not installed correctly directly in the PATH.")
        sys.exit(1)

def check_for_kma():
    """Checks if kma is installed"""
    try:
        subprocess.call(["kma"], stdout=open(os.devnull, 'wb'))
    except FileNotFoundError:
        self.logger.info("kma is not installed correctly directly in the PATH.")
        sys.exit(1)



import os
import sys

def find_best_template_from_spa_file(spa_file, database):
    """Returns the mapping results from the reference mapping"""
    if os.path.exists(spa_file):
        template_score = 0
        template_number = None
        reference_header_text = ""
        with open(spa_file, 'r') as f:
            data = f.read().split("\n")
        data = data[:-1] #Last line is empty
        for item in data:
            item = item.split("\t")
            if item[0][0] != "#":
                if float(item[2]) > template_score:
                    template_score = float(item[2])
                    template_number = int(item[1])
                    reference_header_text = item[0]
        return template_number, template_score, reference_header_text



import os
import sys
import logging
import multiprocessing
import time

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')] + sys.path

from mintyper.validateInput import validate_input
from mintyper.kma import KMARunner
from mintyper.findBestTemplate import find_best_template_from_spa_file
import mintyper.ccphylo as ccphylo
def mintyper_pipeline(arguments):
    if not os.path.exists(arguments.output):
        os.makedirs(arguments.output)

    logging.basicConfig(
        format='%(asctime)s %(message)s',
        filename=arguments.output + '/mintyper.log',
        level=logging.INFO)
    validate_input(arguments)
    logging.info('Input validated. Mintyper is running with the following parameters:')
    logging.info(arguments)

    all_input_files_string = ' '.join(arguments.illumina + arguments.nanopore + arguments.iontorrent)

    threads = int(multiprocessing.cpu_count()/2)

    if arguments.reference != None:
        arguments.database = arguments.output + '/tmp_db'
        template_number = 1
        logging.info('Reference provided: {}'.format(arguments.reference))
        KMARunner(arguments.reference,
                  arguments.database,
                  '',
                  '').index()
        with open(arguments.output + '/tmp_db.name') as f:
            reference_header_text = f.readline().strip()
    else:
        KMARunner(all_input_files_string,
                  arguments.output + '/read_mapping',
                  arguments.database,
                  '-mem_mode -Sparse -ss c -t {}'.format(threads)).run()
        template_number, template_score, reference_header_text = find_best_template_from_spa_file(arguments.output + '/read_mapping.spa', arguments.database)
        logging.info('Best template found: {}'.format(reference_header_text))

    os.system('mkdir {}/alignments'.format(arguments.output))

    if arguments.nanopore != []:
        for item in arguments.nanopore:
            prefix = item.split('/')[-1].split('.')[0]
            KMARunner(item,
                      arguments.output + '/alignments/' + prefix,
                      arguments.database,
                      '-mint3 -Mt1 {} -t {} -vcf'.format(template_number, threads)).run()
    if arguments.iontorrent != []:
        for item in arguments.iontorrent:
            prefix = item.split('/')[-1].split('.')[0]
            KMARunner(item,
                      arguments.output + '/alignments/' + prefix,
                      arguments.database,
                      '-mint2 -Mt1 {} -t {} -vcf'.format(template_number, threads)).run()

    if arguments.illumina != []:
        for i in range(0, len(arguments.illumina), 2):
            prefix = arguments.illumina[i].split('/')[-1].split('.')[0]
            KMARunner(arguments.illumina[i] + ' ' + arguments.illumina[i+1],
                      arguments.output + '/alignments/' + prefix,
                      arguments.database,
                      '-mint2 -Mt1 {} -t {} -vcf'.format(template_number, threads)).run()

    time.sleep(3) #CCphylo might crash unless this. not sure why.

    ccphylo_flag = 1
    if arguments.pairwise == True:
        ccphylo_flag = 2
    if arguments.insig_prune == True:
        ccphylo_flag += 32

    ccphylo.CcphyloDist(arguments.output,
                        reference_header_text,
                        ccphylo_flag,
                        arguments.prune_distance,
                        arguments.masking_scheme).run()

    if arguments.cluster_length > 0:
        ccphylo.CcphyloDBSCAN(arguments.output, arguments.cluster_length).run()

    ccphylo.CcphyloTree(arguments.output).run()

    cmd = "cat {}/alignments/*.vcf.gz > {}/combined.vcf.gz" \
        .format(arguments.output, arguments.output)
    os.system(cmd)








import os
import sys
import logging
import subprocess

class CcphyloTrim():
    def __init__(self, target_dir, reference_header_text, ccphylo_flag, prune_distance, masking_motif_file):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        self.target_dir = target_dir
        self.reference_header_text = reference_header_text
        self.ccphylo_flag = ccphylo_flag
        self.prune_distance = prune_distance
        self.masking_motif_file = masking_motif_file
        self.prepare_list_of_alignment_files()

    def prepare_list_of_alignment_files(self):
        """Returns a list of alignment files"""
        run_list = []
        fsa_list = os.listdir(self.target_dir + '/alignments/')
        for item in fsa_list:
            if item.endswith(".fsa"):
                if os.path.getsize(self.target_dir + '/alignments/' + item) > 0:  # non empty alignment
                    run_list.append(self.target_dir + '/alignments/' + item)
                else:
                    logging.info(
                        'The alignment file {} is empty and therefore it was excluded from the analysis'.format(item))
        self.alignment_string = " ".join(run_list)

    def run(self):
        """runs ccphylo"""
        cmd = "ccphylo trim --input {} --reference \"{}\""\
            .format(self.alignment_string, self.reference_header_text)
        if self.prune_distance != None:
            cmd += " --proximity {}".format(self.prune_distance)
        if self.masking_motif_file != None:
            cmd += " --methylation_motifs {}".format(self.masking_motif_file)
        cmd += ' > {}/alignments/multiple_alignment.fsa'.format(self.target_dir)
        self.logger.info(cmd)
        os.system(cmd)


class CcphyloDist():
    def __init__(self, target_dir, reference_header_text, ccphylo_flag, prune_distance, masking_motif_file):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        #self.check_for_ccphylo()
        self.target_dir = target_dir
        self.reference_header_text = reference_header_text
        self.ccphylo_flag = ccphylo_flag
        self.prune_distance = prune_distance
        self.masking_motif_file = masking_motif_file
        self.prepare_list_of_alignment_files()


    def prepare_list_of_alignment_files(self):
        """Returns a list of alignment files"""
        run_list = []
        fsa_list = os.listdir(self.target_dir + '/alignments/')
        for item in fsa_list:
            if item.endswith(".fsa"):
                if os.path.getsize(self.target_dir + '/alignments/' + item) > 0:  # non empty alignment
                    run_list.append(self.target_dir + '/alignments/' + item)
                else:
                    logging.info(
                        'The alignment file {} is empty and therefore it was excluded from the analysis'.format(item))
        self.alignment_string = " ".join(run_list)

    def run(self):
        """runs ccphylo"""
        cmd = "ccphylo dist --input {} --output {}/distmatrix.txt --reference \"{}\"" \
              " --min_cov 1 --normalization_weight 0" \
            .format(self.alignment_string, self.target_dir, self.reference_header_text)
        if self.prune_distance != None:
            cmd += " --proximity {}".format(self.prune_distance)
        if self.masking_motif_file != None:
            cmd += " --methylation_motifs {}".format(self.masking_motif_file)
        if self.ccphylo_flag != 1:
            cmd += " -f {} 2>&1".format(self.ccphylo_flag)
        else:
            cmd += " 2>&1"
        self.logger.info(cmd)
        os.system(cmd)


class CcphyloTree():
    def __init__(self, target_dir):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        self.target_dir = target_dir
        self.prepare_list_of_alignment_files()

    def prepare_list_of_alignment_files(self):
        """Returns a list of alignment files"""
        run_list = []
        fsa_list = os.listdir(self.target_dir + '/alignments/')
        for item in fsa_list:
            if item.endswith(".fsa"):
                if os.path.getsize(self.target_dir + '/alignments/' + item) > 0:  # non empty alignment
                    run_list.append(self.target_dir + '/alignments/' + item)
                else:
                    logging.info(
                        'The alignment file {} is empty and therefore it was excluded from the analysis'.format(item))
        self.alignment_string = " ".join(run_list)

    def run(self):
        """runs ccphylo tree"""
        cmd = "ccphylo tree --input {}/distmatrix.txt --output {}/tree.newick" \
            .format(self.target_dir, self.target_dir)
        self.logger.info(cmd)
        os.system(cmd)

class CcphyloDBSCAN():
    def __init__(self, target_dir, cluster_length):
        self.logger = logging.getLogger(__name__)
        self.logger.setLevel(logging.INFO)
        self.target_dir = target_dir
        self.cluster_length = cluster_length
        self.prepare_list_of_alignment_files()

    def prepare_list_of_alignment_files(self):
        """Returns a list of alignment files"""
        run_list = []
        fsa_list = os.listdir(self.target_dir + '/alignments/')
        for item in fsa_list:
            if item.endswith(".fsa"):
                if os.path.getsize(self.target_dir + '/alignments/' + item) > 0:  # non empty alignment
                    run_list.append(self.target_dir + '/alignments/' + item)
                else:
                    logging.info(
                        'The alignment file {} is empty and therefore it was excluded from the analysis'.format(item))
        self.alignment_string = " ".join(run_list)

    def run(self):
        """runs ccphylo dbscan"""
        cmd = "ccphylo dbscan --max_distance {} --input {}{} --output {}{}" \
            .format(self.cluster_length, self.target_dir, '/distmatrix.txt', self.target_dir, '/clusters.txt')
        #self.logger.info("Running ccphylo with the following command: {}".format(cmd))
        os.system(cmd)






from setuptools import setup, find_packages
from pathlib import Path
import os
import sys

from mintyper.version import __version__

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

sys.path = [os.path.join(os.path.dirname(os.path.realpath(__file__)),'..')] + sys.path


setup(
    name='mintyper',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version=__version__,
    packages=find_packages(),
    data_files=[],
    include_package_data=True,
    url='https://https://github.com/MBHallgren/mintyper',
    license='',
    install_requires=(),
    author='Malte B. Hallgren',
    scripts=['bin/mintyper'],
    author_email='malhal@food.dtu.dk',
    description='mintyper: an outbreak-detection method for accurate and rapid SNP'
                ' typing of clonal clusters with noisy long reads.'
)